%
%
%

To generate the display function code and its surrounding support
definitions & declarations, the per-backend varying bits have been
factored out into a @NameMapping@. The various NameMapping values
we use are declared below.

\begin{code}
module NameMapping where

import Types
import CCompile ( CCode, Imports, showCDoc, showCOp, showURTOp,
		  ppImportVar
		)
import DExp ( Op )
import PP
\end{code}

\begin{code}
data NameMapping
 = NameMapping
    { ret_type :: String
    , fun_name :: String
    , fun_name_unqual :: String
    , prelude  :: String
    , cleanUp  :: String
    , initCode :: String -> Imports -> String
    , postlude :: String  -> Bool -> String
    , type_map :: (Type -> String)
    , op_map   :: (Op   -> String)
    }
\end{code}

\begin{code}
cMapping :: NameMapping
cMapping =
   NameMapping
     { ret_type = "void"
     , prelude  = "#include <malloc.h>\n#include \"PanPlugin.h\"\n"
     , initCode = \ nm is  -> showCDoc (ppInitDisplay nm "Displayer" ("init_" ++ nm) "()" "()" is)
     , postlude = \ _ _ -> ""
     , fun_name = "sample"
     , fun_name_unqual = "sample"
     , type_map = showCType
     , op_map   = showCOp
     , cleanUp  = ""
     }

cppMapping :: NameMapping
cppMapping =
   NameMapping
     { ret_type = "STDMETHODIMP"
     , prelude  = unlines
                    [ "#include <malloc.h>"
		    , "#include <atlbase.h>"
		    , "extern CComModule _Module;"
		    , "#include <atlcom.h>"
--		    , "#include \"PanObj.h\""
		    , "#include \"uiSpec.h\""
		    , "class CPanImageImpl : "
	            , "     public CComObjectRootEx<CComSingleThreadModel>,"
--		    , "     public PanImageDisplay,"
--		    , "     public PanImageUI,"
		    , "     public IPanImage,"
		    , "     public IPanImageUI,"
		    , "     public IntUpdater,"
		    , "     public FloatUpdater"
		    , "{"
		    , "public:"
		    , "BEGIN_COM_MAP(CPanImageImpl)"
		    , "   COM_INTERFACE_ENTRY(IPanImage)"
		    , "   COM_INTERFACE_ENTRY(IPanImageUI)"
		    , "END_COM_MAP()"
		    ]
     , initCode = \ _ is -> showCDoc (ppDecls (map initImg is))
     , postlude = \ _ _  -> "};"
     , fun_name = "Display"
     , fun_name_unqual = "Display"
     , type_map = showCType
     , op_map   = showCOp
     , cleanUp  = "return S_OK;"
     }

constraintMapping :: NameMapping
constraintMapping = cppMapping{type_map = showTy}
 where
   showTy :: Type -> String
   showTy t = 
     case t of
       Double -> "RVar"
       _      -> error ("constraintMapping cannot handle: " ++ show t)


compMapping :: NameMapping
compMapping =
   NameMapping
     { ret_type = "STDMETHODIMP"
     , prelude  = ""
     , initCode = \ _ is -> showCDoc (ppDecls (map initImg is))
     , postlude = \ _ _  -> ""
     , fun_name = "Display"
     , fun_name_unqual = "Display"
     , type_map = showCType
     , op_map   = showCOp
     , cleanUp  = "return S_OK;"
     }


-- C++ DXTransform mapping
dxtMapping :: NameMapping
dxtMapping =
   NameMapping
     { ret_type = "void"
     , prelude  = ""
     , initCode = \ _ is -> showCDoc (ppDecls (map initImg is))
     , postlude = \ _ _ -> ""
     , fun_name = "CPanImageImpl::Display"
     , fun_name_unqual = "Display"
     , type_map = showCType
     , op_map   = showCOp
     , cleanUp  = ""
     }


--C (Photoshop) mapping
cFilterMapping :: NameMapping
cFilterMapping =
   NameMapping
     { ret_type = "void"
     , prelude  = unlines
                    [ "#include <malloc.h>"
		    , "#include \"uiSpec.h\""
		    , "#include \"PanPlugin.h\""
		    , "class CPanImageImpl : "
		    , "     public PanImagePhSDisplay,"
		    , "     public IntUpdater,"
		    , "     public FloatUpdater"
		    , "{"
		    , "public:"
		    ]
     , initCode = \ _ is -> showCDoc (ppDecls (map initImg is))
     , postlude =  \ _ _ ->
     	     unlines
		[ "};"
		, "PanImagePhSDisplay* mkDisplay(void* pParams) {"
		, "   CPanImageImpl* pObj = new CPanImageImpl();"
		, "   pObj->SetParamPtr(pParams);"
		, "   return pObj;"
	        , "}"
		]

     , fun_name = "Display"
     , fun_name_unqual = "Display"
     , type_map = showCType
     , op_map   = showCOp
     , cleanUp  = ""
     }

--C (AVI filter) mapping
cAVIFilterMapping :: NameMapping
cAVIFilterMapping =
   NameMapping
     { ret_type = "void"
     , prelude  = "#include <malloc.h>\n#include \"PanPlugin.h\"\n"
     , initCode = \ _ is ->  showCDoc (ppImgInit "FilterDisplayer" is)
     , postlude = \ nm varying ->
     		     "static int timeVarying = " ++ (if varying then "1;" else "0;")
     , fun_name = "display"
     , fun_name_unqual = "display"
     , type_map = showCType
     , op_map   = showCOp
     , cleanUp  = ""
     }

-- COM+ 
urtMapping :: NameMapping
urtMapping =
   NameMapping
     { ret_type = "void"
     , prelude  = unlines
                    [ "#import \"SeeImage.obj\""
                    , "class DispPlugin : public Displayer {"
                    , "public:"
                    ]
     , initCode = \ _ is -> showCDoc (ppDecls (map initImg is))
     , postlude = \ _ varying ->
                    let flg = if varying then "true" else "false" in
                    unlines
		     [ "};"
                     , "int main(int argc, char** argv)"
                     , "{"
                     , "  SeeImage seeImg = new SeeImage(0);"
                     , "  DispPlugin disp = new DispPlugin();"
                     , "  seeImg.SetDisplayer(disp," ++ flg ++ ");"
                     , "  seeImg.WinMain();"
                     , "  return 0;"
                     , "}"
                     ]

     , fun_name = "displayer"
     , fun_name_unqual = "displayer"
     , type_map = showCType
     , op_map   = showURTOp
     , cleanUp  = ""
     }

\end{code}

\begin{code}
ppInitDisplay :: String -> String -> String -> String -> String -> Imports -> CCode
ppInitDisplay nm retType funName paramDecl params ls = 
  text "" $$
  ifCpp (text "extern \"C\"") $$
  text retType <+> text funName <> text paramDecl $$
  text "{" $$
  text "initParams" <> text params <> semi $$
  ppDecls (map initImg ls) $$
  text "return" <+> text nm <> semi $$
  text "}" $$
  text ""

initImg (nm,prefix, n)  = 
   ppImportVar prefix n <+> equals <+> 
   text "LoadImportedImg" <> ppTuple [ text (show nm), text "NULL", text "NULL" ]

{-
 Emit code that declares and initializes import images.
-}
ppImgInit :: String -> Imports -> CCode
ppImgInit dispCTy ls = 
  text "" $$
  ifCpp (text "extern \"C\"") $$  -- Matches one in PhSlib.lib sources
  text dispCTy <+> text "mkDisplay()" $$
  text "{" $$
  ppDecls (map initImg ls) $$
  text "return &display;" $$
  text "}" $$
  text ""

ifCpp d = 
  text "#ifdef __cplusplus" $$
  d $$
  text "#endif"
\end{code}
